#!/usr/local/bin/ruby

require "find"
require "cgi"
require "erb"

require "yar"

class CGI
   def method_missing(name)
      params[name.to_s][0]
   end
end

class ArchiveFile
   attr_reader :machine, :remoteid, :localid, :date
   def initialize( machine, remoteid, localid, date )
      @machine, @localid, @remoteid, @date = machine, localid, remoteid, date
      nil unless File.exist?(self.to_s)
   end
   def to_s
      File.join( @machine, @remoteid, @date + "-" + @localid + ".dat" )
   end
   def self.find_files( dir = "." )
      Dir.glob("#{dir}/**/*.dat").map{|f|
         dirname = File.dirname( File.dirname( f ) )
         basename = File.basename( File.dirname( f ) )
         /^(\d{8})-(\w+)\.dat$/ =~ File.basename( f )
         self.new( dirname, basename, $2, $1 ) || nil
      }.compact.sort_by{|e| [e.date, e.remoteid, e.machine, e.localid] }.reverse
   end
   def content
      YahooArchive.new(localid).decode(open(self.to_s))
   end
   def uri_param
      [:machine, :remoteid, :localid, :date].map{|s| "#{s}=#{self.send(s)}" }.join(";")
   end
end

class String
   def format_message_html
      CGI.escapeHTML(self.strip.sub(/^<font[^>]+>/,"")).gsub(/(https?:\/\/[-_.!~*'()a-zA-Z0-9;\/?:@&=+,%\#\$]+)/, '<a href="\1">\1</a>')
   end
end

begin
   if File.readable?( "yim-conf.yml" )
      require "yaml"
      @conf = YAML::load( open( "yim-conf.yml" ) )
   end
   @cgi = CGI.new
   if @cgi.date
      file = ArchiveFile.new(@cgi.machine,
                             @cgi.remoteid,
                             @cgi.localid,
                             @cgi.date)
      result = ERB.new( open("yim.f.html"){|f| f.read } ).result(binding)
   elsif @cgi.search and not @cgi.q.empty?
      search_result = []
      require "nkf"
      ArchiveFile.find_files.each do |f|
         f.content.each do |time, user, message|
            if /#{@cgi.q}/i =~ NKF.nkf("-wZ1", message)
               search_result << [ time,
                                  user,
                                  message,
                                  f
                                ]
            end
         end
      end
      result = ERB.new( open("yim.search.html"){|f| f.read } ).result(binding)
   else
      files = ArchiveFile.find_files
      result = ERB.new( open("yim.html"){|f| f.read } ).result(binding)
   end
   print @cgi.header({"charset" => "utf-8"})
   puts result
rescue
   if @cgi
      print @cgi.header( 'type' => 'text/plain' )
   else
      print "Content-Type: text/plain\n\n"
   end
   puts "#$! (#{$!.class})"
   puts ""
   puts $@.join( "\n" )
end
